// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <string.h>
#include "../../plugins/fit/fit.h"
#include "../../sim.h"
double lib_error(struct fitvars *config,char *name,char * path)
{
int i=0;
int exp_data_len=4;
int numbers[exp_data_len];
double mul_value[exp_data_len];
numbers[0]=18;
numbers[1]=24;
numbers[2]=30;
numbers[3]=37;

char exp_name[exp_data_len][40];
char sim_name[exp_data_len][40];
char out_name_exp[exp_data_len][1000];
char out_name_sim[exp_data_len][1000];
char out_name_delta[exp_data_len][1000];
char config_file[1000];
char tempstring[1000];
double stop=0.0;
double dh=0.0;
struct istruct exp[exp_data_len];
struct istruct sim[exp_data_len];
struct istruct mul_data;
double start=0.0;
double error=0.0;

sprintf(exp_name[0],"%s./exp/%s/10.dat",path,name);
sprintf(exp_name[1],"%s./exp/%s/20.dat",path,name);
sprintf(exp_name[2],"%s./exp/%s/40.dat",path,name);
sprintf(exp_name[3],"%s./exp/%s/80.dat",path,name);

for (i=0;i<exp_data_len;i++)
{
	sprintf(sim_name[i],"%s./sim/%s/snapshots/pl_lam_%d.dat",path,name,numbers[i]);
	sprintf(out_name_exp[i],"./error_%s_exp_%d.dat",name,i);
	sprintf(out_name_sim[i],"./error_%s_sim_%d.dat",name,i);
	sprintf(out_name_delta[i],"./error_%s_delta_%d.dat",name,i);

}

sprintf(config_file,"%s./exp/%s/fit.inp",path,name);

FILE *info=fopen(config_file,"r");
if (info==NULL)
{
	printf("Problem with %s\n",config_file);
	getchar();
}
fscanf(info,"%s",tempstring);
fscanf(info,"%le",&dh);
fscanf(info,"%s",tempstring);
fscanf(info,"%le",&(mul_value[0]));
fscanf(info,"%s",tempstring);
fscanf(info,"%le",&(mul_value[1]));
fscanf(info,"%s",tempstring);
fscanf(info,"%le",&(mul_value[2]));
fscanf(info,"%s",tempstring);
fscanf(info,"%le",&(mul_value[3]));
fscanf(info,"%s",tempstring);
if (strcmp("#end",tempstring)!=0)
{
	printf("Problem with fit.inp in %s!\n","./exp/pulse_voc/");
	getchar();
}
fclose(info);




for (i=0;i<exp_data_len;i++)
{
	inter_load(&(exp[i]),exp_name[i]);
	inter_load(&(sim[i]),sim_name[i]);

	inter_sort(&(exp[i]));
	inter_sort(&(sim[i]));

}

inter_init_mesh(&mul_data,100,start,stop);

double max_exp=inter_get_max(&(exp[0]));
double max_sim=inter_get_max(&(sim[0]));
error=0.0;
double local_error=0.0;

for (i=0;i<exp_data_len;i++)
{
	inter_set_value(&mul_data,1.0);

	inter_div_double(&(exp[i]),max_exp/mul_value[i]);
	inter_div_double(&(sim[i]),max_sim/mul_value[i]);

	start=exp[i].x[0];
	if (start<sim[i].x[0]) start=sim[i].x[0];

	stop=exp[i].x[exp[i].len-1];
	if (stop>sim[i].x[sim[i].len-1]) stop=sim[i].x[sim[i].len-1];

	inter_save(&exp[i],out_name_exp[i]);
	inter_save(&sim[i],out_name_sim[i]);

	local_error=inter_get_diff(out_name_delta[i],&(exp[i]),&(sim[i]),start,stop,&mul_data);
	error+=local_error;

	printf("pl error %d %le\n",i,local_error);
}

for (i=0;i<exp_data_len;i++)
{
	inter_free(&(exp[i]));
	inter_free(&(sim[i]));
}

inter_free(&mul_data);

return error;
}
