// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <errno.h>
#include <util.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include <true_false.h>
#include "dump_ctrl.h"
#include "light.h"
#include "buffer.h"

void light_dump(struct light *in)
{
char out_dir[1024];
if ((get_dump_status(dump_optics_verbose)==TRUE)&&(in->Gn[0]!=0.0))
{
	sprintf(out_dir,"%s/light_dump/",in->output_path);
	struct stat st = {0};

	if (stat(out_dir, &st) == -1)
	{
	}

	FILE *out;
	int i;
	int ii;

	out=fopena(out_dir,"light_2d_Ep.dat","w");
	for (i=0;i<in->lpoints;i++)
	{
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le %le\n",in->l[i],in->x[ii],pow(pow(in->Ep[i][ii],2.0)+pow(in->Epz[i][ii],2.0),0.5));

		}

	fprintf(out,"\n");
	}
	fclose(out);

	out=fopena(out_dir,"light_2d_En.dat","w");
	for (i=0;i<in->lpoints;i++)
	{
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le %le\n",in->l[i],in->x[ii],pow(pow(in->En[i][ii],2.0)+pow(in->Enz[i][ii],2.0),0.5));
		}

	fprintf(out,"\n");
	}
	fclose(out);

	out=fopena(out_dir,"light_2d_E_mod.dat","w");
	for (i=0;i<in->lpoints;i++)
	{
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le %le\n",in->l[i],in->x[ii],pow(pow(in->Ep[i][ii]+in->En[i][ii],2.0)+pow(in->Enz[i][ii]+in->Epz[i][ii],2.0),1.0));
		}

	fprintf(out,"\n");
	}
	fclose(out);

	out=fopena(out_dir,"light_2d_photons.dat","w");
	for (i=0;i<in->lpoints;i++)
	{
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le %le\n",in->l[i],in->x[ii],in->photons[i][ii]);
		}

	fprintf(out,"\n");
	}
	fclose(out);

	out=fopena(out_dir,"light_2d_photons_asb.dat","w");
	for (i=0;i<in->lpoints;i++)
	{
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le %le\n",in->l[i],in->x[ii],in->photons_asb[i][ii]);
		}

	fprintf(out,"\n");
	}
	fclose(out);


	out=fopena(out_dir,"light_2d_n.dat","w");
	for (i=0;i<in->lpoints;i++)
	{
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le %le\n",in->l[i],in->x[ii],in->n[i][ii]);
		}

	fprintf(out,"\n");
	}
	fclose(out);

	out=fopena(out_dir,"light_lambda_sun.dat","w");
	for (i=0;i<in->lpoints;i++)
	{
		fprintf(out,"%le %le\n",in->l[i],in->sun[i]);
	}
	fclose(out);

	out=fopena(out_dir,"light_lambda_sun_norm.dat","w");
	for (i=0;i<in->lpoints;i++)
	{
		fprintf(out,"%le %le\n",in->l[i],in->sun_norm[i]);
	}
	fclose(out);

	out=fopena(out_dir,"light_lambda_sun_photons.dat","w");
	for (i=0;i<in->lpoints;i++)
	{
		fprintf(out,"%le %le\n",in->l[i],in->sun_photons[i]);
	}
	fclose(out);

	out=fopena(out_dir,"light_lambda_alpha.dat","w");
	for (i=0;i<in->lpoints;i++)
	{
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le %le\n",in->l[i],in->x[ii],in->alpha[i][ii]);
		}

	fprintf(out,"\n");
	}
	fclose(out);
}
}

void light_dump_1d(struct light *in, int i,char *ext)
{
char out_dir[1024];
char line[1024];
char out_name[200];
struct buffer data_photons;
struct buffer data_photons_norm;
struct buffer data_light_1d_Ep;
struct buffer data_light_1d_En;
struct buffer data_pointing;
struct buffer data_E_tot;
struct buffer data_1d_photons_tot;
struct buffer data_r;
struct buffer data_t;
struct buffer buf;

buffer_init(&data_photons);
buffer_init(&data_photons_norm);
buffer_init(&data_light_1d_Ep);
buffer_init(&data_light_1d_En);
buffer_init(&data_pointing);
buffer_init(&data_E_tot);
buffer_init(&data_1d_photons_tot);
buffer_init(&data_r);
buffer_init(&data_t);
buffer_init(&buf);

if ((get_dump_status(dump_optics)==TRUE)&&(in->sun_E[i]!=0.0))
{

	sprintf(out_dir,"%s/light_dump/",in->output_path);
	struct stat st = {0};

	if (stat(out_dir, &st) == -1)
	{
	}


	FILE *out;
	FILE *out2;
	int ii;
	char name[400];
	double max=0.0;

	if (i==0)
	{
		max=inter_array_get_max(in->photons_tot,in->points);

		buffer_malloc(&data_1d_photons_tot);
		buffer_malloc(&buf);
		buf.y_mul=1.0;
		buf.x_mul=1e9;
		strcpy(buf.title,"Normalized photon density");
		strcpy(buf.type,"xy");
		strcpy(buf.x_label,"Position");
		strcpy(buf.y_label,"Hole current density");
		strcpy(buf.x_units,"$nm$");
		strcpy(buf.y_units,"$a.u.$");
		buf.logscale_x=0;
		buf.logscale_y=0;
		buffer_add_info(&buf);

		for (ii=0;ii<in->points;ii++)
		{
			sprintf(line,"%le %le\n",in->x[ii]-in->device_start,in->photons_tot[ii]/max);
			buffer_add_string(&buf,line);
		}

		sprintf(out_name,"light_1d_photons_tot_norm%s.dat",ext);
		buffer_dump(out_dir,out_name,&buf);
		buffer_free(&buf);


		char name_1d_photons_tot[200];
		sprintf(name_1d_photons_tot,"light_1d_photons_tot%s.dat",ext);



		for (ii=0;ii<in->points;ii++)
		{
			sprintf(line,"%le %le\n",in->x[ii]-in->device_start,in->photons_tot[ii]);
			buffer_add_string(&data_1d_photons_tot,line);

		}

		buffer_dump(out_dir,name_1d_photons_tot,&data_1d_photons_tot);
	}

	buffer_malloc(&data_photons);
	buffer_malloc(&data_photons_norm);
	buffer_malloc(&data_light_1d_Ep);
	buffer_malloc(&data_light_1d_En);
	buffer_malloc(&data_pointing);
	buffer_malloc(&data_E_tot);
	buffer_malloc(&data_r);
	buffer_malloc(&data_t);

	char name_photons[200];
	char name_photons_norm[200];
	char name_light_1d_Ep[200];
	char name_light_1d_En[200];
	char name_pointing[200];
	char name_E_tot[200];
	char name_r[200];
	char name_t[200];

	sprintf(name_photons,"light_1d_%.0f_photons%s.dat",in->l[i]*1e9,ext);
	sprintf(name_photons_norm,"light_1d_%.0f_photons%s_norm.dat",in->l[i]*1e9,ext);
	sprintf(name_light_1d_Ep,"light_1d_%.0f_Ep%s.dat",in->l[i]*1e9,ext);
	sprintf(name_light_1d_En,"light_1d_%.0f_En%s.dat",in->l[i]*1e9,ext);
	sprintf(name_pointing,"light_1d_%.0f_pointing%s.dat",in->l[i]*1e9,ext);
	sprintf(name_E_tot,"light_1d_%.0f_E_tot%s.dat",in->l[i]*1e9,ext);
	sprintf(name_r,"light_1d_%.0f_r%s.dat",in->l[i]*1e9,ext);
	sprintf(name_t,"light_1d_%.0f_t%s.dat",in->l[i]*1e9,ext);

	max=inter_array_get_max(in->photons[i],in->points);
	for (ii=0;ii<in->points;ii++)
	{
		sprintf(line,"%le %le\n",in->x[ii],in->photons[i][ii]);
		buffer_add_string(&data_photons,line);

		sprintf(line,"%le %le\n",in->x[ii]-in->device_start,in->photons[i][ii]/max);
		buffer_add_string(&data_photons_norm,line);

		sprintf(line,"%le %le %le %le\n",in->x[ii],pow(pow(in->Ep[i][ii],2.0)+pow(in->Epz[i][ii],2.0),0.5),in->Ep[i][ii],in->Epz[i][ii]);
		buffer_add_string(&data_light_1d_Ep,line);

		sprintf(line,"%le %le %le %le\n",in->x[ii],pow(pow(in->En[i][ii],2.0)+pow(in->Enz[i][ii],2.0),0.5),in->En[i][ii],in->Enz[i][ii]);
		buffer_add_string(&data_light_1d_En,line);

		sprintf(line,"%le %le\n",in->x[ii],in->pointing_vector[i][ii]);
		buffer_add_string(&data_pointing,line);

		sprintf(line,"%le %le %le\n",in->x[ii],in->E_tot_r[i][ii],in->E_tot_i[i][ii]);
		buffer_add_string(&data_E_tot,line);

		sprintf(line,"%le %le %le %le\n",in->x[ii],cabs(in->r[i][ii]),creal(in->r[i][ii]),cimag(in->r[i][ii]));
		buffer_add_string(&data_r,line);

		sprintf(line,"%le %le %le %le\n",in->x[ii],cabs(in->t[i][ii]),creal(in->t[i][ii]),cimag(in->t[i][ii]));
		buffer_add_string(&data_t,line);
	}

	buffer_dump(out_dir,name_photons,&data_photons);
	buffer_dump(out_dir,name_photons_norm,&data_photons_norm);
	buffer_dump(out_dir,name_light_1d_Ep,&data_light_1d_Ep);
	buffer_dump(out_dir,name_light_1d_En,&data_light_1d_En);
	buffer_dump(out_dir,name_pointing,&data_pointing);
	buffer_dump(out_dir,name_E_tot,&data_E_tot);
	buffer_dump(out_dir,name_r,&data_r);
	buffer_dump(out_dir,name_t,&data_t);




	buffer_free(&data_photons);
	buffer_free(&data_photons_norm);
	buffer_free(&data_light_1d_Ep);
	buffer_free(&data_light_1d_En);
	buffer_free(&data_pointing);
	buffer_free(&data_E_tot);
	buffer_free(&data_r);
	buffer_free(&data_t);

	if (i==0)
	{
		buffer_free(&data_1d_photons_tot);
	}

	if (get_dump_status(dump_optics_verbose)==TRUE)
	{
		sprintf(name,"%s/light_1d_%.0f_layer%s.dat",out_dir,in->l[i]*1e9,ext);
		out=fopen(name,"w");
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %d\n",in->x[ii],in->layer[ii]);
		}
		fclose(out);

		sprintf(name,"%s/light_1d_%.0f_Gn%s.dat",out_dir,in->l[i]*1e9,ext);
		out=fopen(name,"w");
		sprintf(name,"%s/light_1d_%.0f_Gn%s_norm.dat",out_dir,in->l[i]*1e9,ext);
		out2=fopen(name,"w");
		max=inter_array_get_max(in->Gn,in->points);
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le\n",in->x[ii]-in->device_start,in->Gn[ii]);
			fprintf(out2,"%le %le\n",in->x[ii]-in->device_start,in->Gn[ii]/max);
		}
		fclose(out);
		fclose(out2);


		sprintf(name,"%s/light_1d_%.0f_Gp%s.dat",out_dir,in->l[i]*1e9,ext);
		out=fopen(name,"w");
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le\n",in->x[ii],in->Gp[ii]);
		}
		fclose(out);





		sprintf(name,"%s/light_1d_%.0f_E%s.dat",out_dir,in->l[i]*1e9,ext);
		out=fopen(name,"w");
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le\n",in->x[ii],pow(pow(in->Ep[i][ii]+in->En[i][ii],2.0)+pow(in->Enz[i][ii]+in->Epz[i][ii],2.0),0.5));
		}
		fclose(out);



		sprintf(name,"%s/light_1d_%.0f_t%s.dat",out_dir,in->l[i]*1e9,ext);
		out=fopen(name,"w");
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le\n",in->x[ii],cabs(in->t[i][ii]));
		}
		fclose(out);

		sprintf(name,"%s/light_1d_%.0f_r%s.dat",out_dir,in->l[i]*1e9,ext);
		out=fopen(name,"w");
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le\n",in->x[ii],cabs(in->r[i][ii]));
		}
		fclose(out);

		sprintf(name,"%s/light_1d_%.0f_photons_abs%s.dat",out_dir,in->l[i]*1e9,ext);
		out=fopen(name,"w");
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le\n",in->x[ii],in->photons_asb[i][ii]);
		}
		fclose(out);


		sprintf(name,"%s/light_1d_%.0f_n%s.dat",out_dir,in->l[i]*1e9,ext);
		out=fopen(name,"w");

		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le\n",in->x[ii],in->n[i][ii]);
		}


		fclose(out);

		sprintf(name,"%s/light_1d_%.0f_alpha%s.dat",out_dir,in->l[i]*1e9,ext);
		out=fopen(name,"w");

		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le\n",in->x[ii],in->alpha[i][ii]);
		}


		fclose(out);


		out=fopena(out_dir,"light_sun_wavelength_E.dat","w");
		for (ii=0;ii<in->lpoints;ii++)
		{
			fprintf(out,"%le %le\n",in->l[ii],in->sun_E[ii]);
		}
		fclose(out);

	}



}

}

