// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "../../sim.h"
#include "../../dump.h"
#include "../../ntricks.h"
#include "../../dynamic_store.h"
#include "pulse.h"

static int unused __attribute__((unused));

struct pulse pulse_config;

void sim_pulse(struct device *in)
{
pulse_load_config(&pulse_config,in);
//<clean>
in->dt=pulse_config.pulse_dt;



in->go_time=FALSE;


//struct istruct pulseout;
//inter_init(&pulseout);
in->externalv=pulse_config.pulse_Vexternal;
printf("Running pulse. %e\n",in->externalv);
int ittr=0;

int step=0;
light_solve_and_update(in,&(in->mylight),in->Psun,0.0,0.0);

//ramp(in,in->Vapplied,pulse_config.pulse_Vexternal,100.0);
sim_externalv(in,pulse_config.pulse_Vexternal);
in->Vapplied=pulse_config.pulse_Vexternal;
double V=in->Vapplied;
double V0=pulse_config.pulse_Vexternal;
printf("%le %le\n",V,V0);
light_solve_and_update(in,&(in->mylight),in->Psun,0.0,0.0);

newton_pulse(in,V0,FALSE);
device_timestep(in);
newton_pulse(in,V0,FALSE);
device_timestep(in);
in->time=0.0;

time_init(in);

device_timestep(in);

in->go_time=TRUE;


//double np_out_tot=0.0;
double t0=pulse_config.pulse_t0;
double t1=pulse_config.pulse_t0+pulse_config.pulse_t1;
double t2=pulse_config.pulse_t0+pulse_config.pulse_t1+pulse_config.pulse_t2;
FILE *pulse_out=fopen("./pulse_j.dat","w");
fclose(pulse_out);

pulse_out=fopen("./pulse_v.dat","w");
fclose(pulse_out);

pulse_out=fopen("./pulse_nleft.dat","w");
fclose(pulse_out);

pulse_out=fopen("./pulse_rtot.dat","w");
fclose(pulse_out);

double local_time=in->time;

struct dynamic_store store;
dump_dynamic_init(&store,in);

struct istruct out_i;
inter_init(&out_i);

struct istruct out_v;
inter_init(&out_v);

struct istruct out_mue;
inter_init(&out_mue);

struct istruct out_muh;
inter_init(&out_muh);

struct istruct out_mue_c;
inter_init(&out_mue_c);

struct istruct out_muh_c;
inter_init(&out_muh_c);

struct istruct out_G;
inter_init(&out_G);

struct istruct lost_charge;
inter_init(&lost_charge);


double extracted_through_contacts=0.0;
carrier_count_reset(in);
reset_np_save(in);
do
{



	if (local_time<t0)
	{
		if (pulse_config.pulse_light==TRUE)
		{
			light_solve_and_update(in,&(in->mylight),0.0,0.0,0.0);
		}else
		{
			V=V0;

			if ((local_time>=pulse_config.pulse_laser_on)&&(local_time<=(pulse_config.pulse_laser_on+pulse_config.pulse_laser_width)))
			{
				light_solve_and_update(in,&(in->mylight),in->Psun,pulse_config.pulse_laser_power,pulse_config.pulse_laser_width);
			}else
			{
				light_solve_and_update(in,&(in->mylight),in->Psun,0.0,0.0);
			}

		}

		extracted_through_contacts=0.0;
	}else
	if (local_time<t1)
	{
		if (pulse_config.pulse_light==TRUE)
		{
			light_solve_and_update(in,&(in->mylight),pulse_config.pulse_sun,0.0,0.0);
		}else
		{
		V=pulse_config.pulse_Vss;
		}
	}else
	{
		if (pulse_config.pulse_light==TRUE)
		{
			light_solve_and_update(in,&(in->mylight),0.0,0.0,0.0);
		}else
		{
		V=V0;
		}
	}


	//solve_all(in);

	dump_dynamic_add_data(&store,in,in->time);

	double i0=newton_pulse(in,V,TRUE);


	if (get_dump_status(dump_print_text)==TRUE)
	{
		printf("pulse time=%e step=%d %.1e ",in->time,step,in->last_error);
		printf("Vtot=%lf current = %e mA (%e A/m^2)\n",V,get_I(in)/1e-3,get_J(in));
	}

	ittr++;

	dump_write_to_disk(in);

	plot_now(in,"pulse.plot");


	//inter_append(&pulseout,in->time,fabs(get_I(in)));

	inter_append(&out_i,in->time,i0);
	inter_append(&out_v,in->time,V);
	inter_append(&out_mue,in->time,get_avg_mue(in));
	inter_append(&out_muh,in->time,get_avg_muh(in));
	inter_append(&out_mue_c,in->time,in->mun[0]);
	inter_append(&out_muh_c,in->time,in->mup[0]);
	inter_append(&out_G,in->time,in->Gn[0]);
	inter_append(&lost_charge,in->time,extracted_through_contacts-fabs(get_extracted_n(in)+get_extracted_p(in))/2.0);

	if ((in->time>pulse_config.pulse_bigmeshstart)&&(in->time<pulse_config.pulse_bigmeshstop))
	{
		in->dt=pulse_config.pulse_bigdt;
	}else
	{
		in->dt=pulse_config.pulse_dt;
	}

	device_timestep(in);
	step++;

	if (in->time>(((double)pulse_config.pulse_repeat)*t2)) break;
	if (local_time>t2) local_time=0.0;


	local_time+=in->dt;

	if (in->stop==TRUE)
	{
		break;

	}


}while(1);
char outpath[1000];
struct istruct out_flip;

dump_dynamic_save(in->outputpath,&store);
dump_dynamic_free(&store);

inter_copy(&out_flip,&out_i,TRUE);
inter_mul(&out_flip,-1.0);
sprintf(outpath,"%s%s",in->outputpath,"pulse_i_pos.dat");
inter_save(&out_flip,outpath);

inter_free(&out_flip);


sprintf(outpath,"%s%s",in->outputpath,"pulse_i.dat");
inter_save(&out_i,outpath);

inter_add_x(&out_i,-t0+pulse_config.pulse_shift);
sprintf(outpath,"%s%s",in->outputpath,"pulse_i_norm.dat");
inter_mod(&out_i);
inter_save(&out_i,outpath);


sprintf(outpath,"%s%s",in->outputpath,"pulse_v.dat");
inter_save(&out_v,outpath);

sprintf(outpath,"%s%s",in->outputpath,"pulse_mue.dat");
inter_save(&out_mue,outpath);

sprintf(outpath,"%s%s",in->outputpath,"pulse_muh.dat");
inter_save(&out_muh,outpath);

sprintf(outpath,"%s%s",in->outputpath,"pulse_mue_c.dat");
inter_save(&out_mue_c,outpath);

sprintf(outpath,"%s%s",in->outputpath,"pulse_muh_c.dat");
inter_save(&out_muh_c,outpath);

sprintf(outpath,"%s%s",in->outputpath,"pulse_G.dat");
inter_save(&out_G,outpath);

sprintf(outpath,"%s%s",in->outputpath,"pulse_lost_charge.dat");
inter_save(&lost_charge,outpath);



in->go_time=FALSE;

inter_free(&out_G);
inter_free(&out_muh);
inter_free(&out_mue);
inter_free(&out_muh_c);
inter_free(&out_mue_c);
inter_free(&out_i);
inter_free(&out_v);
//</clean>
}

void pulse_load_config(struct pulse *in,struct device *dev)
{
double ver=0.0;
char name[200];
FILE *config=fopena(dev->outputpath,"pulse.inp","r");

config_read_line_to_double(&(in->pulse_dt),config,"#pulse_dt");

config_read_line_to_double(&(in->pulse_t0),config,"#pulse_t0");

config_read_line_to_double(&(in->pulse_t1),config,"#pulse_t1");

config_read_line_to_double(&(in->pulse_t2),config,"#pulse_t2");

config_read_line_to_double(&(in->pulse_Vexternal),config,"#pulse_Vexternal");

config_read_line_to_int(&(in->pulse_repeat),config,"#pulse_repeat");

config_read_line_to_double(&(in->Rshort_pulse),config,"#Rshort_pulse");

config_read_line_to_double(&(in->pulse_laser_on),config,"#pulse_laser_on");

config_read_line_to_double(&(in->pulse_laser_width),config,"#pulse_laser_width");

config_read_line_to_double(&(in->pulse_laser_power),config,"#pulse_laser_power");

config_read_line_to_double(&(in->pulse_Vss),config,"#pulse_Vss");

config_read_line_to_int(&(in->pulse_light),config,"#pulse_light");

config_read_line_to_double(&(in->pulse_sun),config,"#pulse_sun");

config_read_line_to_double(&(in->pulse_shift),config,"#pulse_shift");

config_read_line_to_double(&(in->pulse_bigmeshstart),config,"#pulse_bigmeshstart");

config_read_line_to_double(&(in->pulse_bigmeshstop),config,"#pulse_bigmeshstop");

config_read_line_to_double(&(in->pulse_bigdt),config,"#pulse_bigdt");

config_read_line_to_double(&(ver),config,"#ver");

if (ver!=1.2)
{
ewe("File compatability problem %s\n","pulse.inp");
}

unused=fscanf(config,"%s",name);
if (strcmp("#end",name)!=0)
{
ewe("Problem with pulse.inp file last item read %s!\n",name);
}
fclose(config);
}
