// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "../../dump.h"
#include "../../sim.h"
#include "stark.h"

static double glob_wanted=0.0;
static int glob_use_cap=0;

void newton_aux_stark(struct device *in,double V,double* i,double* didv,double* didphi,double* didxil,double* didxipl,double* didphir,double* didxir,double* didxipr)
{
/*
double C=0.0;
double L=0.0;
C=in->C;
L=0.0;
if (glob_use_cap==FALSE)
{
	C=0.0;
	L=0.0;
}
double i0=*i;
double didv0=*didv;
double didphi0=*didphi;
double didxil0=*didxil;
double didxipl0=*didxipl;
double didphir0=*didphir;
double didxir0=*didxir;
double didxipr0=*didxipr;
double itot=i0+C*(V-in->Vapplied_last)/in->dt+V/in->Rshunt;
double ditotdv=didv0+C*(1.0)/in->dt+1.0/in->Rshunt;
//
*i=glob_wanted+L*(itot-in->ilast)/in->dt-(V+(in->Rcontact)*(itot));
printf("%le %le %le\n",glob_wanted,L*(itot-in->ilast)/in->dt,(V+(in->Rcontact)*(itot)));
*didv=L*(ditotdv)/in->dt-(1.0+(in->Rcontact)*(ditotdv));
*didphi=L*(didphi0)/in->dt-((in->Rcontact)*(didphi0));
*didxil=L*(didxil0)/in->dt-((in->Rcontact)*(didxil0));
*didxipl=L*(didxipl0)/in->dt-((in->Rcontact)*(didxipl0));
*didphir=L*(didphir0)/in->dt-((in->Rcontact)*(didphir0));
*didxir=L*(didxir0)/in->dt-((in->Rcontact)*(didxir0));
*didxipr=L*(didxipr0)/in->dt-((in->Rcontact)*(didxipr0));
*/
//<clean>
double C=in->C;
if (glob_use_cap==FALSE) C=0.0;
double i0=*i;
double didv0=*didv;
double didphi0=*didphi;
double didxil0=*didxil;
double didxipl0=*didxipl;
double didphir0=*didphir;
double didxir0=*didxir;
double didxipr0=*didxipr;

*i=glob_wanted-(V+((in->Rcontact)*(i0+V/in->Rshunt+C*(V-in->Vapplied_last)/in->dt)));
*didv=-(1.0+((in->Rcontact)*(didv0+1.0/in->Rshunt+C*(1.0)/in->dt)));
*didphi=-((in->Rcontact)*didphi0);
*didxil=-((in->Rcontact)*didxil0);
*didxipl=-((in->Rcontact)*didxipl0);
*didphir=-((in->Rcontact)*didphir0);
*didxir=-((in->Rcontact)*didxir0);
*didxipr=-((in->Rcontact)*didxipr0);


//</clean>
return;
}

double newton_sim_stark(struct device *in,double Vtot,int usecap)
{
double C=in->C;
if (usecap==FALSE)
{
	C=0.0;
}

if (stark_config.stark_sim_contacts==TRUE)
{
	in->kl_in_newton=TRUE;
	solver_realloc(in);
	glob_wanted=Vtot;
	glob_use_cap=usecap;
	in->newton_aux=&newton_aux_stark;
	solve_all(in);
}else
{
	newton_sim_stark_simple(in);
	in->kl_in_newton=FALSE;
	solver_realloc(in);
	solve_all(in);
}

return get_I(in)+C*(in->Vapplied-in->Vapplied_last)/in->dt+in->Vapplied/in->Rshunt;
}

void newton_sim_stark_simple(struct device *in)
{
in->kl_in_newton=FALSE;
solver_realloc(in);
solve_all(in);
return;
}
